@@ -36,7 +36,7 @@
 #include <gtk/gtk.h>
 #include <gedit/gedit-utils.h>
 #include "gedit-spell-checker-dialog.h"
-#include "gedit-spell-checker-dialog-marshal.h"
+#include "gedit-spell-marshal.h"
 
 struct _GeditSpellCheckerDialog 
 {
@@ -77,12 +77,6 @@ enum
 	NUM_COLUMNS
 };
 
-static void	gedit_spell_checker_dialog_class_init 		(GeditSpellCheckerDialogClass * klass);
-static void	gedit_spell_checker_dialog_init 		(GeditSpellCheckerDialog *spell_checker);
-static void	gedit_spell_checker_dialog_destroy 		(GtkObject *object);
-/*
-static void 	gedit_spell_checker_dialog_finalize 		(GObject *object);
-*/
 static void	update_suggestions_list_model 			(GeditSpellCheckerDialog *dlg, 
 								 GSList *suggestions);
 
@@ -110,35 +104,28 @@ static void	suggestions_list_row_activated_handler		(GtkTreeView *view,
 								 GeditSpellCheckerDialog *dlg);
 
 
-static GtkWindowClass *parent_class = NULL;
 static guint signals [LAST_SIGNAL] = { 0 };
 
-GType
-gedit_spell_checker_dialog_get_type (void)
+G_DEFINE_TYPE(GeditSpellCheckerDialog, gedit_spell_checker_dialog, GTK_TYPE_WINDOW)
+
+static void
+gedit_spell_checker_dialog_destroy (GtkObject *object)
 {
-	static GType gedit_spell_checker_dialog_type = 0;
+	GeditSpellCheckerDialog *dlg = GEDIT_SPELL_CHECKER_DIALOG (object);
 
-	if(!gedit_spell_checker_dialog_type) 
+	if (dlg->spell_checker != NULL)
 	{
-		static const GTypeInfo gedit_spell_checker_dialog_info = 
-		{
-			sizeof (GeditSpellCheckerDialogClass),
-			NULL, /* base init */
-			NULL, /* base finalize */
-			(GClassInitFunc) gedit_spell_checker_dialog_class_init, /* class init */
-			NULL, /* class finalize */
-			NULL, /* class data */
-			sizeof (GeditSpellCheckerDialog),
-			0,
-			(GInstanceInitFunc) gedit_spell_checker_dialog_init
-		};
-
-		gedit_spell_checker_dialog_type = g_type_register_static (GTK_TYPE_WINDOW,
-							"GeditSpellCheckerDialog",
-							&gedit_spell_checker_dialog_info, 0);
+		g_object_unref (dlg->spell_checker);
+		dlg->spell_checker = NULL;
 	}
 
-	return gedit_spell_checker_dialog_type;
+	if (dlg->misspelled_word != NULL)
+	{
+		g_free (dlg->misspelled_word);
+		dlg->misspelled_word = NULL;
+	}
+
+	GTK_OBJECT_CLASS (gedit_spell_checker_dialog_parent_class)->destroy (object);
 }
 
 static void
@@ -148,15 +135,9 @@ gedit_spell_checker_dialog_class_init (GeditSpellCheckerDialogClass * klass)
 
 	object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
-	/*
-	object_class->finalize = gedit_spell_checker_dialog_finalize;
-	*/
-
 	GTK_OBJECT_CLASS (object_class)->destroy = gedit_spell_checker_dialog_destroy;
 
-	signals [IGNORE] = 
+	signals[IGNORE] = 
 		g_signal_new ("ignore",
  			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -167,7 +148,7 @@ gedit_spell_checker_dialog_class_init (GeditSpellCheckerDialogClass * klass)
 			      1, 
 			      G_TYPE_STRING);
 
-	signals [IGNORE_ALL] = 
+	signals[IGNORE_ALL] = 
 		g_signal_new ("ignore_all",
  			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -178,7 +159,7 @@ gedit_spell_checker_dialog_class_init (GeditSpellCheckerDialogClass * klass)
 			      1, 
 			      G_TYPE_STRING);
 
-	signals [CHANGE] = 
+	signals[CHANGE] = 
 		g_signal_new ("change",
  			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -190,7 +171,7 @@ gedit_spell_checker_dialog_class_init (GeditSpellCheckerDialogClass * klass)
 			      G_TYPE_STRING,
 			      G_TYPE_STRING);
 	
-	signals [CHANGE_ALL] = 
+	signals[CHANGE_ALL] = 
 		g_signal_new ("change_all",
  			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -202,7 +183,7 @@ gedit_spell_checker_dialog_class_init (GeditSpellCheckerDialogClass * klass)
 			      G_TYPE_STRING,
 			      G_TYPE_STRING);
 
-	signals [ADD_WORD_TO_PERSONAL] = 
+	signals[ADD_WORD_TO_PERSONAL] = 
 		g_signal_new ("add_word_to_personal",
  			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -215,22 +196,35 @@ gedit_spell_checker_dialog_class_init (GeditSpellCheckerDialogClass * klass)
 }
 
 static void
-gedit_spell_checker_dialog_init (GeditSpellCheckerDialog *dlg)
+create_dialog (GeditSpellCheckerDialog *dlg,
+	       const gchar *data_dir)
 {
 	GtkWidget *error_widget;
 	GtkWidget *content;
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *cell;
 	GtkTreeSelection *selection;
+	gchar *root_objects[] = {
+		"content",
+		"check_word_image",
+		"add_word_image",
+                "ignore_image",
+                "change_image",
+                "ignore_all_image",
+                "change_all_image",
+		NULL
+	};
 	gboolean ret;
-
+	gchar *ui_file;
+	
 	g_return_if_fail (dlg != NULL);
 
 	dlg->spell_checker = NULL;
 	dlg->misspelled_word = NULL;
 
-	ret = gedit_utils_get_glade_widgets (GEDIT_GLADEDIR "spell-checker.glade2",
-		"content",
+	ui_file = g_build_filename (data_dir, "spell-checker.ui", NULL);
+	ret = gedit_utils_get_ui_objects (ui_file,
+		root_objects,
 		&error_widget,
 
 		"content", &content,
@@ -246,14 +240,15 @@ gedit_spell_checker_dialog_init (GeditSpellCheckerDialog *dlg)
 		"suggestions_list", &dlg->suggestions_list,
 		"language_label", &dlg->language_label,
 		NULL);
+	g_free (ui_file);
 
 	if (!ret)
 	{
 		gtk_widget_show (error_widget);
-			
-		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dlg)->vbox),
-					     error_widget);
-		
+
+		gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dlg))),
+				    error_widget, TRUE, TRUE, 0);
+
 		return;
 	}
 
@@ -269,6 +264,7 @@ gedit_spell_checker_dialog_init (GeditSpellCheckerDialog *dlg)
 	gtk_label_set_label (GTK_LABEL (dlg->language_label), "");
 			
 	gtk_container_add (GTK_CONTAINER (dlg), content);
+	g_object_unref (content);
 
 	gtk_window_set_resizable (GTK_WINDOW (dlg), FALSE);
 	gtk_window_set_title (GTK_WINDOW (dlg), _("Check Spelling"));
@@ -288,12 +284,11 @@ gedit_spell_checker_dialog_init (GeditSpellCheckerDialog *dlg)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (dlg->suggestions_list), column);
 
 	gtk_tree_view_set_search_column (GTK_TREE_VIEW (dlg->suggestions_list),
-			COLUMN_SUGGESTIONS);
+					 COLUMN_SUGGESTIONS);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dlg->suggestions_list));
 
-	gtk_tree_selection_set_mode (selection,
-				   GTK_SELECTION_SINGLE);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 
 	/* Set default button */
 	GTK_WIDGET_SET_FLAGS (dlg->change_button, GTK_CAN_DEFAULT);
@@ -302,60 +297,36 @@ gedit_spell_checker_dialog_init (GeditSpellCheckerDialog *dlg)
 	gtk_entry_set_activates_default (GTK_ENTRY (dlg->word_entry), TRUE);
 
 	/* Connect signals */
-	g_signal_connect (G_OBJECT (dlg->word_entry), "changed",
+	g_signal_connect (dlg->word_entry, "changed",
 			  G_CALLBACK (word_entry_changed_handler), dlg);
-	g_signal_connect (G_OBJECT (dlg->close_button), "clicked",
+	g_signal_connect (dlg->close_button, "clicked",
 			  G_CALLBACK (close_button_clicked_handler), dlg);
 	g_signal_connect (selection, "changed", 
 			  G_CALLBACK (suggestions_list_selection_changed_handler), 
 			  dlg);
-	g_signal_connect (G_OBJECT (dlg->check_word_button), "clicked",
+	g_signal_connect (dlg->check_word_button, "clicked",
 			  G_CALLBACK (check_word_button_clicked_handler), dlg);
-	g_signal_connect (G_OBJECT (dlg->add_word_button), "clicked",
+	g_signal_connect (dlg->add_word_button, "clicked",
 			  G_CALLBACK (add_word_button_clicked_handler), dlg);
-	
-	g_signal_connect (G_OBJECT (dlg->ignore_button), "clicked",
+	g_signal_connect (dlg->ignore_button, "clicked",
 			  G_CALLBACK (ignore_button_clicked_handler), dlg);
-	g_signal_connect (G_OBJECT (dlg->ignore_all_button), "clicked",
+	g_signal_connect (dlg->ignore_all_button, "clicked",
 			  G_CALLBACK (ignore_all_button_clicked_handler), dlg);
-	g_signal_connect (G_OBJECT (dlg->change_button), "clicked",
+	g_signal_connect (dlg->change_button, "clicked",
 			  G_CALLBACK (change_button_clicked_handler), dlg);
-	g_signal_connect (G_OBJECT (dlg->change_all_button), "clicked",
+	g_signal_connect (dlg->change_all_button, "clicked",
 			  G_CALLBACK (change_all_button_clicked_handler), dlg);
-
-	g_signal_connect (G_OBJECT (dlg->suggestions_list), "row-activated",
+	g_signal_connect (dlg->suggestions_list, "row-activated",
 			  G_CALLBACK (suggestions_list_row_activated_handler), dlg);
 }
 
 static void
-gedit_spell_checker_dialog_destroy (GtkObject *object)
-{
-	GeditSpellCheckerDialog *dlg = GEDIT_SPELL_CHECKER_DIALOG (object);
-
-	if (dlg->spell_checker != NULL)
-	{
-		g_object_unref (dlg->spell_checker);
-		dlg->spell_checker = NULL;
-	}
-
-	if (dlg->misspelled_word != NULL)
-	{
-		g_free (dlg->misspelled_word);
-		dlg->misspelled_word = NULL;
-	}
-
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
-}
-
-/*
-static void 
-gedit_spell_checker_dialog_finalize (GObject *object)
+gedit_spell_checker_dialog_init (GeditSpellCheckerDialog *dlg)
 {
 }
-*/
 
 GtkWidget *
-gedit_spell_checker_dialog_new (void)
+gedit_spell_checker_dialog_new (const gchar *data_dir)
 {
 	GeditSpellCheckerDialog *dlg;
 
@@ -363,12 +334,15 @@ gedit_spell_checker_dialog_new (void)
 			g_object_new (GEDIT_TYPE_SPELL_CHECKER_DIALOG, NULL));
 
 	g_return_val_if_fail (dlg != NULL, NULL);
+	
+	create_dialog (dlg, data_dir);
 
 	return GTK_WIDGET (dlg);
 }
 
 GtkWidget *
-gedit_spell_checker_dialog_new_from_spell_checker (GeditSpellChecker *spell)
+gedit_spell_checker_dialog_new_from_spell_checker (GeditSpellChecker *spell,
+						   const gchar *data_dir)
 {
 	GeditSpellCheckerDialog *dlg;
 
@@ -378,6 +352,8 @@ gedit_spell_checker_dialog_new_from_spell_checker (GeditSpellChecker *spell)
 			g_object_new (GEDIT_TYPE_SPELL_CHECKER_DIALOG, NULL));
 
 	g_return_val_if_fail (dlg != NULL, NULL);
+	
+	create_dialog (dlg, data_dir);
 
 	gedit_spell_checker_dialog_set_spell_checker (dlg, spell);
 
